from flask import Blueprint, jsonify, request
from models.base import Session
from models.message import Message
import logging
from ..middleware.auth import auth_required
from datetime import datetime

logger = logging.getLogger(__name__)

bp = Blueprint('message_api', __name__)

@bp.route("/api/dashboard/messages", methods=['GET'])
@auth_required(roles=['admin', 'user'])
def get_messages():
    try:
        page = int(request.args.get('page', 1))
        per_page = int(request.args.get('per_page', 20))

        # Get messages with pagination
        messages, total_messages, total_chats = Message.get_messages(page, per_page)

        # Log the first message to help debug
        if messages and len(messages) > 0:
            logger.info(f"First message data: {messages[0]}")

        return jsonify({
            "status": "success",
            "data": messages,
            "pagination": {
                "page": page,
                "per_page": per_page,
                "total": total_messages,
                "total_pages": (total_messages + per_page - 1) // per_page,
                "total_chats": total_chats
            }
        })
    except Exception as e:
        logger.error(f"Error getting messages: {str(e)}")
        return jsonify({
            "status": "error",
            "message": str(e)
        }), 500

@bp.route("/api/dashboard/messages/<message_id>", methods=['GET'])
@auth_required(roles=['admin', 'user'])
def get_message(message_id):
    try:
        session = Session()
        try:
            # Try to convert to int for numeric IDs
            message_id_int = int(message_id)
            message = session.query(Message).get(message_id_int)

            if not message:
                logger.warning(f"Message not found with ID: {message_id}")
                return jsonify({
                    "status": "error",
                    "message": "Message not found"
                }), 404

            # Convert to dict while session is still open
            message_dict = message.to_dict()

            return jsonify({
                "status": "success",
                "data": message_dict
            })
        except (ValueError, TypeError):
            # If not a numeric ID, return an error
            logger.error(f"Invalid message ID format: {message_id}")
            return jsonify({
                "status": "error",
                "message": "Invalid message ID format"
            }), 400
    except Exception as e:
        logger.error(f"Error getting message: {str(e)}")
        return jsonify({
            "status": "error",
            "message": str(e)
        }), 500
    finally:
        session.close()

@bp.route("/api/dashboard/messages/<message_id>", methods=['PUT'])
@auth_required(roles=['admin'])
def update_message(message_id):
    try:
        data = request.json
        logger.info(f"Received update request for message {message_id} with data: {data}")

        if not data:
            return jsonify({
                "status": "error",
                "message": "No data provided"
            }), 400

        session = Session()
        try:
            # Try to convert to int for numeric IDs
            message_id_int = int(message_id)
            message = session.query(Message).get(message_id_int)

            if not message:
                logger.warning(f"Message not found for update: {message_id}")
                return jsonify({
                    "status": "error",
                    "message": "Message not found"
                }), 404

            # Close the session since we'll use the model's update_message method
            session.close()
            session = None

            # Use the model's update_message method to update the message
            # This method handles setting the last_updated timestamp
            updated_message_obj = Message.update_message(message_id_int, data)

            if not updated_message_obj:
                return jsonify({
                    "status": "error",
                    "message": "Failed to update message"
                }), 500

            # Get the updated message as a dictionary
            updated_message = updated_message_obj.to_dict()

            logger.info(f"Successfully updated message {message_id}")
            return jsonify({
                "status": "success",
                "data": updated_message
            })
        except (ValueError, TypeError):
            # If not a numeric ID, return an error
            logger.error(f"Invalid message ID format: {message_id}")
            return jsonify({
                "status": "error",
                "message": "Invalid message ID format"
            }), 400
    except Exception as e:
        logger.error(f"Error updating message: {str(e)}")
        return jsonify({
            "status": "error",
            "message": str(e)
        }), 500
    finally:
        session.close()

@bp.route("/api/dashboard/messages", methods=['POST'])
@auth_required(roles=['admin'])
def create_message():
    try:
        data = request.json
        logger.info(f"Received create message request with data: {data}")

        if not data:
            return jsonify({
                "status": "error",
                "message": "No data provided"
            }), 400

        # Validate required fields
        required_fields = ['message_type', 'chat_id']
        missing_fields = [field for field in required_fields if field not in data]
        if missing_fields:
            return jsonify({
                "status": "error",
                "message": f"Missing required fields: {', '.join(missing_fields)}"
            }), 400

        # Create the message
        message = Message.create_message(data)

        return jsonify({
            "status": "success",
            "data": message.to_dict(),
            "message": "Message created successfully"
        }), 201
    except Exception as e:
        logger.error(f"Error creating message: {str(e)}")
        return jsonify({
            "status": "error",
            "message": str(e)
        }), 500

@bp.route("/api/dashboard/messages/<message_id>", methods=['DELETE'])
@auth_required(roles=['admin'])
def delete_message(message_id):
    try:
        session = Session()
        try:
            # Try to convert to int for numeric IDs
            message_id_int = int(message_id)
            message = session.query(Message).get(message_id_int)

            if not message:
                logger.warning(f"Message not found for deletion: {message_id}")
                return jsonify({
                    "status": "error",
                    "message": "Message not found"
                }), 404

            # Delete the message
            session.delete(message)
            session.commit()

            logger.info(f"Successfully deleted message {message_id}")
            return jsonify({
                "status": "success",
                "message": "Message deleted successfully"
            })
        except (ValueError, TypeError):
            # If not a numeric ID, return an error
            logger.error(f"Invalid message ID format: {message_id}")
            return jsonify({
                "status": "error",
                "message": "Invalid message ID format"
            }), 400
    except Exception as e:
        logger.error(f"Error deleting message: {str(e)}")
        return jsonify({
            "status": "error",
            "message": str(e)
        }), 500
    finally:
        session.close()
